 /*
  * file: config.h
  * 
  * PCP - A PPP Control Panel for X11 with OSF/Motif
  * 
  * Copyright (C) 1996 by Bjoern Fischer <bakerman@hrz.uni-bielefeld.de>
  * Further Improvements by Armin Richter <AR.Richter@ws.rz.tu-bs.de>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  */

/* If you want to have a running pcp you have to adapt this file before
 * compilation to your pppd installation/configuration */

#ifndef config_h
#define config_h
#endif

/* N O T E :      ALL VALUES HAVE TO BE STRINGS, EVEN HUP_SIGNAL AND
 *                LOG_INTERVAL. DO NOT USE A MACRO AS HUP_SIGNAL (e.g.
 *                SIGTERM) AS IT WON'T BE EXPANDED.
 * 
 * PPP_COMMAND:   Shell command which invokes pppd and sets up the
 *                connection.
 *
 * PPP_PARAMS:    Parameters for this command.
 *                Don't forget to quote any '"' as '\"' - 
 *                otherwise the source won't compile. This string may 
 *                contain a single %s, which will be replaced by the 
 *                PHONE_NUMBER string.
 *
 * PS_COMMAND:    Shell command sequence which outputs a string having 
 *                the chat's PID as the first characters followed by a space
 *                to a file which then must be specified in PS_FILE
 *                (usually a
 *                "ps | grep chat | grep ppp.chat |\
 *                 grep -v grep >/var/run/chat.pid")
 *
 * PS_FILE:       see PS_COMMAND
 *
 * WHO_COMMAND:   Shell command sequence which outputs a list of currently
 *                logged in users to a file which then has to specified in
 *                WHO_FILE.
 *
 * WHO_FILE:      See WHO_COMMAND         
 *
 * PHONE_NUMBER:  This string (yes, it may contain non-digits) is inserted
 *                into the PPP_COMMAND string, if it contains a %s format
 *                tag.
 * 
 * PID_FILE:      File which contains the process id of a running pppd.
 *                If you use this utility, I recommended to run pppd 
 *                only once at a time. Later versions may handle more 
 *                than one instance of pppd.
 * 
 * HUP_SIGNAL:    If you press the hangup button, pcp will send this signal
 *                to the running pppd process.
 *                Keep in mind, that pppd - started with the 'persist' 
 *                option - will make a new attempt to dial and connect
 *                when it receives a SIGHUP. Remember to use for a SIGTERM
 *                the numeral 15 and not the macro. This has to
 *                be a string.
 *
 * BRK_SIGNAL:    If you press the break&cleanup button,
 *                pcp will send this signal
 *                to the running pppd & chat processes.
 *                Keep in mind, that pppd - started with the 'persist' 
 *                option - will make a new attempt to dial and connect
 *                when it receives a SIGHUP. Remember to use for a SIGTERM
 *                the numeral 15 and not the macro. This has to
 *                be a string.
 * 
 * LOG_FILE:      You have to Configure syslogd - through /etc/syslog.conf -
 *                to write all
 *                pppd messages into this separate file. This file must have 
 *                been set to mode 644 - at least - to get pcp work
 *                properly, because this is imho the easiest way to get
 *                information from pppd.
 * 
 * LOG_INTERVAL:  the value defines the time in milliseconds between every
 *                update of the pppd log and the animation.
 *                Too small values (>250) won't be 
 *                more useful but would trash the cpu with senseless ....! 
 *                Too big values let you wait for the log-update.
 *                Maybe this is separated in future versions ...
 *                This must be a string, too!
 *
 * TIMER_INTERVAL:the value defines the time in milliseconds between every
 *                update of the clock.
 *                Too small values (>250) won't be 
 *                more useful but would trash the cpu with senseless ....! 
 *                Too big values make the clock stagger.
 *                This must be a string, too!
 *
 * LOCK_FILE:     File where pcp stores its own pid. This is used to
 *                determine if another pcp-session is already running.
 *                Of course, this can only work if the other pcp-task
 *                has the same value for its LOCK_FILE. So better don't
 *                touch this.
 */

#define PPP_COMMAND   "/usr/sbin/pppd"
#define PPP_PARAMS    "modem"
/*#define PPP_PARAMS    "connect '/usr/sbin/chat -v -f /etc/ppp/ppp.chat' \
modem lock noipdefault defaultroute ipcp-accept-local debug mru 3300\
-ac -bsdcomp crtscts /dev/modem 115200 \
disconnect \"chat -- \\d+++\\d\\c OK ath0 OK\" lcp-echo-interval 30 \
lcp-echo-failure 4"*/
#define PS_COMMAND    "ps | grep chat | grep ppp.chat | grep -v \
grep >/var/run/chat.pid"
#define WHO_COMMAND   "who >/var/run/wholog"
#define PS_FILE       "/var/run/chat.pid"
/*
#define PHONE_NUMBER  " "
*/
#define PID_FILE      "/var/run/ppp0.pid"
#define WHO_FILE      "/var/run/wholog"
#define HUP_SIGNAL    "1"                   
#define BRK_SIGNAL    "15"                   
/*#define LOG_FILE      "/var/log/ppplog"*/
#define LOG_FILE      "/var/log/messages"
#define LOG_INTERVAL  "2500"
#define REDIAL_INTERVAL "15000"
#define TIMER_INTERVAL "900"
#define LOCK_FILE     "/var/run/pcp.pid"
