  /*
  * file: pcp.c
  * 
  * PCP - A PPP Control Panel for X11 with OSF/Motif
  * 
  *  Main Part and Animation 
  *  written 1996 by Bjoern Fischer 
  *  <bakerman@hrz.uni-bielefeld.de>
  *  Further Improvements and maintenance by 
  *  Armin Richter <AR.Richter@tu-bs.de>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  PCP Copyright 1996&97&98 Bjoern Fischer and Armin Richter
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  */

#include <time.h>
#include <Xm/XmAll.h>
#include <Xm/XmP.h>
#include <X11/Xmu/Editres.h>
#include <X11/xpm.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/stat.h>
#include "config.h"
#include "anim.xpm"
#ifdef NCURSES
#include <ncurses.h> /* make it textmode-compatible */
#endif

#define APP_CLASS    "Pcp"                                            
#define MAX_ARGS     20
#define MAX_LEN      256

#define BREAKPOINT printf("This is line:%d\n", __LINE__)

/* forwards */
Widget createWidgets(Widget);
void dialButtonCB(Widget, XtPointer, XtPointer);
void redialCB(XtPointer, XtIntervalId*);
void hupButtonCB(Widget, XtPointer, XtPointer);
void exitButtonCB(Widget, XtPointer, XtPointer);
void breakButtonCB(Widget, XtPointer, XtPointer);
void doExitCB(Widget, XtPointer, XtPointer);
void doHupCB(Widget, XtPointer, XtPointer);
void dontHupCB(Widget, XtPointer, XtPointer);
void doRunCB(Widget, XtPointer, XtPointer);
void doBreakCB(Widget, XtPointer, XtPointer);
void dontBreakCB(Widget, XtPointer, XtPointer);
void updateLogCB(XtPointer, XtIntervalId*);
void updatetimerCB(XtPointer, XtIntervalId*);

/* globals */
static Widget         appShell;
static XtAppContext   appContext;
static XtIntervalId   timerhandleredial = 0, timerhandlelog = 0, timerhandletimer = 0;
static Widget         animation;
static int            flic = 0;
static Widget         dialButton;
static Widget         hupButton;
static Widget         breakButton;
static int            pppPID = 0, chatPID = 0;
static Widget         logText;
static Widget         whoText;
static Widget         timer;
static FILE          *logFile = NULL;
static FILE          *lockFile = NULL;
static FILE          *whoFile = NULL;
#if 0
static FILE          *modem = NULL;
#endif
static char          *buffer = NULL;
static char          *whobuffer = NULL;
static char           timerstring[30];
static unsigned long  bufSize;
static unsigned long  whobufSize;
static char          *pppCmdStr, *pppParStr, *psCmdStr, *whoCmdStr, copyright[1600], helptext[1600];
static Pixmap         anim[MAX_ANIM];
static time_t         sessionstart;
static Boolean        connection = FALSE;
static Boolean        cleanstart = FALSE ;                             
static Boolean        stop = FALSE ;
static double         etime;                                           
static unsigned int   h, m;                                            
static time_t         now ;                                            
static Arg            args[1]; /* zum Textndern im Timer */           
static int            updatelog = 1;
static int            updatewho = 1;

/* fallback resources; this prog should work without editing the 
 * Xt resource database */

static String fallbackResources[] = {
   "*title*fontList:            -*-helvetica-bold-r-normal-*-18-*-*-*-*-*-*-*",
   "*dialButton*fontList:       -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*",
   "*hupButton*fontList:        -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*",
   "*exitButton*fontList:       -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*",
   "*breakButton*fontList:      -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*",   
   "*runDlg*messageString:     There seems to be another pcp-session running. Exit this one?",
   "*runDlg*dialogTitle:       Warning!",
   "*runDlg*fontList:          -*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
   "*hupDlg*messageString:      Do you really want to hang up?",          
   "*hupDlg*dialogTitle:        Hang Up?",
   "*hupDlg*fontList:           -*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
   "*brkDlg*messageString:     Do you really want to break?",         
   "*brkDlg*dialogTitle:       Emergency Break?",                     
   "*brkDlg*fontList:          -*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
   "*exitDlg*messageString:     Still online - really quit pcp (connection will remain)?",
   "*exitDlg*dialogTitle:       Warning!",
   "*exitDlg*fontList:          -*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
   NULL
};

/* application resources and Xt command line arguments */

#define XpcpNpppCommand        "pppCommand"
#define XpcpCpppCommand        "PppCommand"
#define XpcpNpppParams         "pppParams"
#define XpcpCpppParams         "PppParams"
#define XpcpNpsCommand         "psCommand"                             
#define XpcpCpsCommand         "PsCommand"                             
#define XpcpNwhoCommand        "whoCommand"                            
#define XpcpCwhoCommand        "WhoCommand"                            
#if 0
#define XpcpNphoneNumber       "phoneNumber"
#define XpcpCphoneNumber       "PhoneNumber"
#endif
#define XpcpNpsFile            "psFile"                                
#define XpcpCpsFile            "PsFile"                                
#define XpcpNpidFile           "pidFile"
#define XpcpCpidFile           "PidFile"
#define XpcpNwhoFile           "whoFile"                                
#define XpcpCwhoFile           "WhoFile"                                
#define XpcpNchatFile          "chatPID"                               
#define XpcpCchatFile          "ChatPID"                               
#define XpcpNhupSignal         "hupSignal"
#define XpcpChupSignal         "HupSignal"
#define XpcpNbrkSignal         "brkSignal"
#define XpcpCbrkSignal         "BrkSignal"
#define XpcpNlogFile           "logFile"
#define XpcpClogFile           "LogFile"
#define XpcpNlogInterval       "logInterval"
#define XpcpClogInterval       "LogInterval"
#define XpcpNredialInterval    "redialInterval"
#define XpcpCredialInterval    "RedialInterval"
#define XpcpNtimerInterval     "timerInterval"
#define XpcpCtimerInterval     "TimerInterval"
#define XpcpNlockFile          "lockFile"
#define XpcpClockFile          "LockFile"
#define XpcpNverbose           "verbose"
#define XpcpCverbose           "Verbose"
#define XpcpNhelp              "help"
#define XpcpChelp              "Help"

typedef struct {
   String  pppCommand;
   String  pppParams;
   String  psCommand;                                                  
   String  whoCommand;                                                  
#if 0
   String  phoneNumber;
#endif
   String  psFile;                                                     
   String  pidFile;
   String  whoFile;
   int     hupSignal;
   int     brkSignal;
   String  logFile;
   int     logInterval;
   int     redialInterval;
   int     timerInterval;
   String  lockFile;
   Boolean verbose;
   Boolean help;
} appRes_t;

static XtResource resources[] = {
     { XpcpNpppCommand, XpcpCpppCommand, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, pppCommand), XmRString, PPP_COMMAND },
     { XpcpNpppParams, XpcpCpppParams, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, pppParams), XmRString, PPP_PARAMS },
     { XpcpNpsCommand, XpcpCpsCommand, XmRString, sizeof(String),      
       XtOffsetOf(appRes_t, psCommand), XmRString, PS_COMMAND },       
     { XpcpNwhoCommand, XpcpCwhoCommand, XmRString, sizeof(String),      
       XtOffsetOf(appRes_t, whoCommand), XmRString, WHO_COMMAND },       
#if 0
     { XpcpNphoneNumber, XpcpCphoneNumber, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, phoneNumber), XmRString, PHONE_NUMBER },
#endif
     { XpcpNpidFile, XpcpCpidFile, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, pidFile), XmRString, PID_FILE },
     { XpcpNpsFile, XpcpCpsFile, XmRString, sizeof(String),            
       XtOffsetOf(appRes_t, psFile), XmRString, PS_FILE },             
     { XpcpNwhoFile, XpcpCwhoFile, XmRString, sizeof(String),            
       XtOffsetOf(appRes_t, whoFile), XmRString, WHO_FILE },             
     { XpcpNhupSignal, XpcpChupSignal, XmRInt, sizeof(int),
       XtOffsetOf(appRes_t, hupSignal), XmRString, HUP_SIGNAL },
     { XpcpNbrkSignal, XpcpCbrkSignal, XmRInt, sizeof(int),
       XtOffsetOf(appRes_t, brkSignal), XmRString, BRK_SIGNAL },
     { XpcpNlogFile, XpcpClogFile, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, logFile), XmRString, LOG_FILE },
     { XpcpNlogInterval, XpcpClogInterval, XmRInt, sizeof(int),
       XtOffsetOf(appRes_t, logInterval), XmRString, LOG_INTERVAL },
     { XpcpNredialInterval, XpcpCredialInterval, XmRInt, sizeof(int),
       XtOffsetOf(appRes_t, redialInterval), XmRString, REDIAL_INTERVAL },
     { XpcpNtimerInterval, XpcpCtimerInterval, XmRInt, sizeof(int),
       XtOffsetOf(appRes_t, timerInterval), XmRString, TIMER_INTERVAL },
     { XpcpNlockFile, XpcpClockFile, XmRString, sizeof(String),
       XtOffsetOf(appRes_t, lockFile), XmRString, LOCK_FILE },
     { XpcpNverbose, XpcpCverbose, XmRBoolean, sizeof(Boolean),
       XtOffsetOf(appRes_t, verbose), XmRString, "False" },
     { XpcpNhelp, XpcpChelp, XmRBoolean, sizeof(Boolean),
       XtOffsetOf(appRes_t, help), XmRString, "False" }
};

static XrmOptionDescRec options[] = {
   { "-c", "."XpcpNpppCommand, XrmoptionSepArg, NULL },
   { "-V", "."XpcpNpppParams, XrmoptionSepArg, NULL },
   { "-C", "."XpcpNpsCommand, XrmoptionSepArg, NULL } ,                
   { "-w", "."XpcpNwhoCommand, XrmoptionSepArg, NULL } ,                
#if 0
   { "-n", "."XpcpNphoneNumber, XrmoptionSepArg, NULL },
#endif
   { "-p", "."XpcpNpidFile, XrmoptionSepArg, NULL },
   { "-P", "."XpcpNpsFile, XrmoptionSepArg, NULL },                    
   { "-W", "."XpcpNwhoFile, XrmoptionSepArg, NULL },                    
   { "-s", "."XpcpNhupSignal, XrmoptionSepArg, NULL },
   { "-b", "."XpcpNbrkSignal, XrmoptionSepArg, NULL },
   { "-l", "."XpcpNlogFile, XrmoptionSepArg, NULL },
   { "-i", "."XpcpNlogInterval, XrmoptionSepArg, NULL },
   { "-r", "."XpcpNredialInterval, XrmoptionSepArg, NULL },
   { "-t", "."XpcpNtimerInterval, XrmoptionSepArg, NULL },
   { "-k", "."XpcpNlockFile, XrmoptionSepArg, NULL },                  
   { "-v", "."XpcpNverbose, XrmoptionNoArg, "True" },
   { "-h", "."XpcpNhelp, XrmoptionNoArg, "True" },
   { "--pppcommand"    , "."XpcpNpppCommand, XrmoptionSepArg, NULL },
   { "--pppparams"     , "."XpcpNpppParams, XrmoptionSepArg, NULL },
   { "--ps"            , "."XpcpNpsCommand, XrmoptionSepArg, NULL } ,        
   { "--who"           , "."XpcpNwhoCommand, XrmoptionSepArg, NULL } ,
#if 0
   { "--phone"         , "."XpcpNphoneNumber, XrmoptionSepArg, NULL },
#endif
   { "--pidfile"       , "."XpcpNpidFile, XrmoptionSepArg, NULL },
   { "--psfile"        , "."XpcpNpsFile, XrmoptionSepArg, NULL }, 
   { "--whofile"       , "."XpcpNwhoFile, XrmoptionSepArg, NULL },            
   { "--hupsignal"     , "."XpcpNhupSignal, XrmoptionSepArg, NULL },
   { "--brksignal"     , "."XpcpNbrkSignal, XrmoptionSepArg, NULL },
   { "--log"           , "."XpcpNlogFile, XrmoptionSepArg, NULL },
   { "--loginterval"   , "."XpcpNlogInterval, XrmoptionSepArg, NULL },
   { "--redialinterval", "."XpcpNredialInterval, XrmoptionSepArg, NULL },
   { "--timerinterval" , "."XpcpNtimerInterval, XrmoptionSepArg, NULL },
   { "--lockfile"      , "."XpcpNlockFile, XrmoptionSepArg, NULL },          
   { "--verbose"       , "."XpcpNverbose, XrmoptionNoArg, "True" },
   { "--help"          , "."XpcpNhelp, XrmoptionNoArg, "True" }
};

static appRes_t appResources;
	
void main (int argc, char **argv)
{
   Widget rundialog;
   /* Widget for "we are already running"-requester */
   Widget mainWindow;
   Widget dw;
   int    i;
   /* open syslog, we simply write our errors and warnings into the
    * system log */
   openlog(argv[0] , LOG_PID, LOG_DAEMON);
 
   /* initialize Xt conveniently */
   appShell = XtAppInitialize (&appContext, APP_CLASS, 
			       options, XtNumber(options),
			       &argc, argv,
			       fallbackResources, (Arg*)NULL, 0);
   /* get application resources */
   XtGetApplicationResources(appShell, &appResources,
			     resources, XtNumber(resources),
			     (Arg*)NULL, 0);

   strcpy(copyright,"*\n");
   strcat(copyright,"*  PCP 0.9.5 Copyright 1996&97&98 by BF and AR\n");
   strcat(copyright,"*\n");
   strcat(copyright,"*  written 1996 by Bjoern Fischer <bakerman@hrz.uni-bielefeld.de>\n");
   strcat(copyright,"*  Further Improvements by Armin Richter <AR.Richter@tu-bs.de>\n");
   strcat(copyright,"*  Please read the documentation, especially the 'SECURITY'-file.\n");
   strcat(copyright,"*\n");
   strcat(copyright,"*  This program is free software; you can redistribute it and/or modify\n");
   strcat(copyright,"*  it under the terms of the GNU General Public License as published by\n");
   strcat(copyright,"*  the Free Software Foundation; either version 2 of the License, or\n");
   strcat(copyright,"*  (at your option) any later version.\n");
   strcat(copyright,"*\n");
   strcat(copyright,"*  This program is distributed in the hope that it will be useful,\n");
   strcat(copyright,"*  but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
   strcat(copyright,"*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
   strcat(copyright,"*  GNU General Public License for more details.\n");
   strcat(copyright,"*\n");
   strcat(copyright,"*  You should have received a copy of the GNU General Public License\n");
   strcat(copyright,"*  along with this program; if not, write to the Free Software\n");
   strcat(copyright,"*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\0");

   /* prepare log buffer */
   buffer = (char*)malloc(bufSize = strlen(copyright)+1);

   /* show interfacespecs */

   if (appResources.help) {
     if (appResources.verbose)
      fprintf(stderr, "%s\n", copyright);
     else
       fprintf(stderr, "This is copyrighted material. Use option -h -v for details.\n\n");

     strcat(helptext,"-c --pppcommand\tcommand_to_start_ppp\n"); 
     strcat(helptext,"-V --pppparams\tparams for this command\n"); 
     strcat(helptext,"-C --ps\tcommand_to_obtain_pid_of_chat\n");
     strcat(helptext,"-w --who\tcommand_to_obtain_logged_in_users\n");
     strcat(helptext,"-W --whofile\tfile_where_these_users_are_stored_by_-w\n");
#if 0
     strcat(helptext,"-n --phone\tphonenumber\n");
#endif
     strcat(helptext,"-p --pidfile\tfile_where_ppppid_is_stored_by_pppd\n");
     strcat(helptext,"-P --psfile\tfile_where_chatpid_shall_be_stored_by_pcp\n");
     strcat(helptext,"-s --hupsignal\tsignal sent by hupbutton\n");
     strcat(helptext,"-b --brksignal\tsignal sent by breakbutton\n");
     strcat(helptext,"-l --log\tlogfile\n");
     strcat(helptext,"-i --loginterval\tinterval_to_actualize_log_etc\n");
     strcat(helptext,"-r --redialinterval\tinterval_to_redial_after_fail\n");
     strcat(helptext,"-t --timerinterval\tinterval_to_actualize_timer_buttons_etc\n");
     strcat(helptext,"-k --lockfile\tpcp_lockfile\n");
     strcat(helptext,"-v --verbose\n");
     strcat(helptext,"-h --help\n\0");

     fprintf(stderr, "%s \n", helptext);
     exit(0);
   }
   
   /* build psCmdStr */                                               
                                                                      
   psCmdStr = malloc(strlen(appResources.psCommand)                   
                     + 1);                                            
   sprintf(psCmdStr, appResources.psCommand);                         

   /* build pppCmdStr */
   
   pppCmdStr = malloc((strlen(appResources.pppCommand) + 1));
   sprintf(pppCmdStr, "%s", appResources.pppCommand);

   /* build pppParStr */
   
   pppParStr = malloc(strlen(appResources.pppParams) + 1);
   sprintf(pppParStr, "%s", appResources.pppParams);

   /* build whoCmdStr */
   
   whoCmdStr = malloc(strlen(appResources.whoCommand) +
		      + 1);
   sprintf(whoCmdStr, appResources.whoCommand);
   
#if 0
   buffer = (char*)malloc(bufSize = strlen(copyright)+1);
#endif
   strcpy(buffer,copyright);

   /* open the log file and move to EOF to pass only the 'freshest'
    * information to the user */
   if ((logFile = fopen(appResources.logFile, "r")) == NULL) 
    {
      fprintf(stderr, "\nCannot open Logfile \"%s\", trying to create...\n", appResources.logFile);
      syslog(LOG_NOTICE, "\nCannot open Logfile \"%s\", trying to create...\n", appResources.logFile);
     if ((logFile = fopen(appResources.logFile, "a")) == NULL)
       {
	 fprintf(stderr, "\nCreation of Logfile \"%s\" failed.\n", appResources.logFile);
	 syslog(LOG_NOTICE, "\nCreation of Logfile \"%s\" failed.\n", appResources.logFile);
       } 
     fclose(logFile);
     if ((logFile = fopen(appResources.logFile, "r")) == NULL) 
       {
	 fprintf(stderr, "\nCannot open Logfile \"%s\", finally failed.\n", appResources.logFile);
	 syslog(LOG_NOTICE, "\nCannot open Logfile \"%s\" finally failed.\n", appResources.logFile);
       }
     else
       fseek(logFile, 0, SEEK_END);
    }
   else
     fseek(logFile, 0, SEEK_END);

   /* Add event handler for editres */
   XtAddEventHandler(appShell, NoEventMask, True,
		     _XEditResCheckMessages, (XtPointer)appShell);

   mainWindow = createWidgets(appShell);

   XtRealizeWidget(appShell);
   
   /* Create animation flics */
   for (i = 0; i < MAX_ANIM; i++)
     XpmCreatePixmapFromData(XtDisplay(appShell), XtWindow(appShell),
 			    anim_xpm[i], &anim[i], 0, 0);

   XtVaSetValues(animation, XmNlabelPixmap, anim[flic], NULL);  

   dw = XmGetXmDisplay(XtDisplay(appShell));
   /* where "shell" is your client's top-level shell. */
   
   XtVaSetValues(dw, XmNdragInitiatorProtocolStyle, XmDRAG_NONE, NULL);
   XtVaSetValues(dw, XmNdragReceiverProtocolStyle,  XmDRAG_NONE, NULL);

   /* check for existing lockfile */
   if ((lockFile = fopen(appResources.lockFile, "r")) == NULL) 
     { 
       /* lockfile erzeugen und Prozess-ID hereinschreiben */
       if ((lockFile = fopen(appResources.lockFile, "w")) != NULL)
	 { 
	   fprintf(lockFile, "%i", (int)getpid());
	   fclose(lockFile);
	 }
     }
   else
     {
       /* Warnung ausgeben */
       rundialog = XmCreateWarningDialog(appShell, "runDlg", NULL, 0);
       XtDestroyWidget(XmMessageBoxGetChild(rundialog, XmDIALOG_HELP_BUTTON));
       XtAddCallback(XmMessageBoxGetChild(rundialog, XmDIALOG_OK_BUTTON),
		     XmNactivateCallback, &doRunCB, (XtPointer)appShell);
       XtManageChild(rundialog);

       syslog(LOG_NOTICE, "\nLockfile \"%s\" already exists! Another pcp-process is running!\nIf this is not true, remove file by hand and retry.\n", appResources.lockFile);
     }

   XtAppMainLoop(appContext);
}

Widget createWidgets(Widget parent)
{
   Widget   form;
   Widget   title;
   Widget   exitButton;
   Widget   separator1;
   XmString tmpString;
   Arg      args[MAX_ARGS];
   Cardinal n;
   
   /* the form widget is nice */
   n = 0;
   XtSetArg(args[n], XmNwidth, 480); n++;
   XtSetArg(args[n], XmNheight, 550); n++;
   form = XmCreateForm(parent, "pcpForm", args, n);

   /* the free animation */
   n = 0;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopOffset, 5); n++;
   XtSetArg(args[n], XmNeditable, FALSE); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightOffset, 5); n++;
   XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
   animation = XmCreateLabel(form, "animation", args, n);
   XtManageChild(animation);
    
   /* the most important title */
   n = 0;
#if 0
   XtSetArg(args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
#endif
   XtSetArg(args[n], XmNrecomputeSize, True ); n++;                    
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;           
   XtSetArg(args[n], XmNleftOffset, 5); n++;                           
#if 0
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;    
   XtSetArg(args[n], XmNrightOffset, 5); n++;                   
#endif
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNtopOffset, 10); n++;
   XtSetArg(args[n], XmNenableUnselectableDrag, FALSE); n++;
   tmpString = XmStringGenerate("PCP - PPP Control Panel", (XmStringTag)NULL,
				XmCHARSET_TEXT, (XmStringTag)NULL);
   XtSetArg(args[n], XmNlabelString, tmpString); n++;
   title = XmCreateLabel(form, "title", args, n);
   XtManageChild(title);
   XmStringFree(tmpString);                                            

   /* Und hier muss jetzt ein Textwidget rein */                       
   n = 0;                                                              
   XtSetArg(args[n], XmNrecomputeSize, True ); n++;                    
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;            
   XtSetArg(args[n], XmNtopOffset, 10); n++;                           
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;          
   XtSetArg(args[n], XmNrightWidget, animation); n++;                  
   XtSetArg(args[n], XmNrightOffset, 5); n++;                          
   tmpString = XmStringGenerate("00:00:00", (XmStringTag)NULL,         
				XmCHARSET_TEXT, (XmStringTag)NULL);    
   XtSetArg(args[n], XmNlabelString, tmpString); n++;                  
   timer = XmCreateLabel(form, "timer", args, n);                      
   XtManageChild(timer);                                               
   XmStringFree(tmpString);                                            

   /* the first separator */                                           
   n = 0;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, animation); n++;
   XtSetArg(args[n], XmNtopOffset, 5); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftOffset, 2); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightOffset, 2); n++;
   XtSetArg(args[n], XmNmargin, 3); n++;
   XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
   separator1 = XmCreateSeparator(form, "separator1", args, n);
   XtManageChild(separator1);

   /* the push buttons */
   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftOffset, 5); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, separator1); n++;
   XtSetArg(args[n], XmNtopOffset, 7); n++;
   XtSetArg(args[n], XmNmarginWidth, 6); n++;
   tmpString = XmStringGenerate("Initiate PPP link", (XmStringTag)NULL,
				XmCHARSET_TEXT, (XmStringTag)NULL);
   XtSetArg(args[n], XmNlabelString, tmpString); n++;
   dialButton = XmCreatePushButton(form, "dialButton", args, n);
   XtManageChild(dialButton);
   XmStringFree(tmpString);

   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNleftWidget, dialButton); n++;
   XtSetArg(args[n], XmNleftOffset, 5); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, separator1); n++;
   XtSetArg(args[n], XmNtopOffset, 7); n++;
   XtSetArg(args[n], XmNmarginWidth, 6); n++;
   XtSetArg(args[n], XmNsensitive, FALSE); n++;
   tmpString = XmStringGenerate("Hang Up", (XmStringTag)NULL,
				XmCHARSET_TEXT, (XmStringTag)NULL);
   XtSetArg(args[n], XmNlabelString, tmpString); n++;
   hupButton = XmCreatePushButton(form, "hupButton", args, n);
   XtManageChild(hupButton);
   XmStringFree(tmpString);

   n = 0;                                                              
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;         
   XtSetArg(args[n], XmNleftWidget, hupButton); n++;                   
   XtSetArg(args[n], XmNleftOffset, 5); n++;                           
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;          
   XtSetArg(args[n], XmNtopWidget, separator1); n++;                   
   XtSetArg(args[n], XmNtopOffset, 7); n++;                            
   XtSetArg(args[n], XmNmarginWidth, 6); n++;                          
   tmpString = XmStringGenerate("Break & Cleanup", (XmStringTag)NULL,  
				XmCHARSET_TEXT, (XmStringTag)NULL);    
   XtSetArg(args[n], XmNlabelString, tmpString); n++;                  
   breakButton = XmCreatePushButton(form, "breakButton", args, n);     
   XtManageChild(breakButton);                                         
   XmStringFree(tmpString);                                            

   n = 0;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightOffset, 5); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, separator1); n++;
   XtSetArg(args[n], XmNtopOffset, 7); n++;
   XtSetArg(args[n], XmNmarginWidth, 6); n++;
   tmpString = XmStringGenerate("Quit", (XmStringTag)NULL,
				XmCHARSET_TEXT, (XmStringTag)NULL);
   XtSetArg(args[n], XmNlabelString, tmpString); n++;
   exitButton = XmCreatePushButton(form, "exitButton", args, n);
   XtManageChild(exitButton);
   XmStringFree(tmpString);

   /* the scrolled text for the system log */
   n = 0;
   XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNeditable,FALSE); n++;
   XtSetArg(args[n], XmNcursorPositionVisible, FALSE); n++;
   XtSetArg(args[n], XmNblinkRate, 0); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftOffset, 5); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightOffset, 5); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, dialButton); n++;
   XtSetArg(args[n], XmNtopOffset, 130); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNbottomOffset, 5); n++;
   logText = XmCreateScrolledText(form, "pppLog", args, n);
   XtManageChild(logText);

   /* the scrolled text for who-output */
   n = 0;
   XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNeditable,FALSE); n++;
   XtSetArg(args[n], XmNcursorPositionVisible, FALSE); n++;
   XtSetArg(args[n], XmNblinkRate, 0); n++;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNleftOffset, 5); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightOffset, 5); n++;
   XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNtopWidget, dialButton); n++;
   XtSetArg(args[n], XmNtopOffset, 10); n++;
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
   XtSetArg(args[n], XmNbottomWidget, logText); n++;
   XtSetArg(args[n], XmNbottomOffset, 5); n++;
   whoText = XmCreateScrolledText(form, "whoText", args, n);
   XtManageChild(whoText);

   /* we don't have many callbacks */
   XtAddCallback(dialButton, XmNactivateCallback, 
		 &dialButtonCB, (XtPointer)appShell);
   XtAddCallback(hupButton, XmNactivateCallback, 
		 &hupButtonCB, (XtPointer)appShell);
   XtAddCallback(exitButton, XmNactivateCallback, 
		 &exitButtonCB, (XtPointer)appShell);
   XtAddCallback(breakButton, XmNactivateCallback,                    
		 &breakButtonCB, (XtPointer)appShell);                    

   /* Change Windowtitle */
   tmpString = XmStringGenerate("PCP 0.9.5 (not connected)", (XmStringTag)NULL,
				XmCHARSET_TEXT, (XmStringTag)NULL);
   XmeSetWMShellTitle(tmpString, appShell);
   XmStringFree(tmpString);
                                                     
   timerhandlelog = 
     XtAppAddTimeOut(appContext, 2500, updateLogCB, (XtPointer)appShell);
   timerhandletimer = 
     XtAppAddTimeOut(appContext, 500, updatetimerCB, (XtPointer)appShell);
   
   XtManageChild(form);
   return(form);
}

void RemoveRedialTimeout ()
{
  if (timerhandleredial)
    {
      XtRemoveTimeOut(timerhandleredial);
      syslog(LOG_INFO, "Redialling stopped...");          
      /* Reinstall other callbacks (Motif seems to make a mess otherwise)*/
      XtRemoveTimeOut(timerhandlelog);
      XtRemoveTimeOut(timerhandletimer);
      timerhandlelog =
	XtAppAddTimeOut(appContext, appResources.logInterval, 
			updateLogCB, (XtPointer)appShell);
      timerhandletimer =
	XtAppAddTimeOut(appContext, appResources.timerInterval, 
			updatetimerCB, (XtPointer)appShell);
    }
}

/* Callback functions */

void dialButtonCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   char timestring[50] ;
   time_t* sessionstartptr ;
   FILE    *psFile;

   if ((pppPID == 0) && (chatPID == 0)) {
     /* delete the log buffer, we're going to start a new session */
     free(buffer);
     buffer = (char*)malloc(bufSize = 1);
     *buffer = '\0';

     /* call shell command to start pppd */
     /* stop redial */
     RemoveRedialTimeout();
     
     /* 'reset' timer */                                                
     sessionstartptr = (time_t*)&sessionstart ;                         
     sessionstart = time(NULL) ;                                        
     cleanstart = TRUE ; /* pppd ist started by this pcp-session */     
     strcpy(timestring, ctime(sessionstartptr)) ;                        
     if (appResources.verbose) {
       syslog(LOG_INFO, "Timerstart: ***> %s <***", timestring);          
       syslog(LOG_INFO, "pppCmdStr: %s %s", pppCmdStr, pppParStr);
     }
     else
       syslog(LOG_INFO, "Timerstart.");          

#if 0
     pppPID = fork(); /* Proze klonen */
     if (pppPID == 0) /* dies hier ist der geklonte Prozess !*/
       {
	 static char *pppargv[2];
	 pppargv[0] = pppCmdStr;
	 char h[255];
	 int i;
	 for (i = 1; i < 10; i++) {
	   scanf(pppParStr, "%s", pppargv[i]);
	   strcpy(h,&pppParStr[strlen(pppargv[i])]); 
	   strcpy(pppParStr,h);	 fprintf(stderr, pppParStr);
	 }
	 pppargv[i++] = NULL;
	 execl(pppCmdStr, pppargv);
#endif
       strcat(pppCmdStr, " ");
       strcat(pppCmdStr, pppParStr);
       system(pppCmdStr);
       system(psCmdStr);                                             
       if ((psFile = fopen(appResources.psFile, "r")) != NULL)
	 if (fscanf(psFile, "%d", &chatPID) == EOF) chatPID = 0;       

#if 0
       exit(0);
       }
     if (pppPID == -1) /* Proze konnte nicht erzeugt werden! */
       {
	 syslog(LOG_ERR, "Could not fork Dialup-Script. Dialup cancelled.");
       }
     syslog(LOG_INFO, "New PID: %i", pppPID);
#endif
     if (stop) stop = FALSE;
     timerhandleredial = 
       XtAppAddTimeOut(appContext, appResources.redialInterval, 
		       redialCB, (XtPointer)appShell);
     sessionstartptr = (time_t*)&sessionstart ;                         
     sessionstart = time(NULL) ;                                        
     strcpy(timestring, ctime(sessionstartptr)) ;                        
     if (appResources.verbose)
       syslog(LOG_INFO, "%s: Redialinterval starts counting now...", timestring);          
   }  
}

void redialCB(XtPointer clientData, XtIntervalId *timerId)

{
   char timestring[50] ;
   time_t* sessionstartptr ;

   /* call shell command to start pppd */
   if (pppPID == 0 && chatPID == 0 && !stop) {
     /* 'reset' timer */                                                
     sessionstartptr = (time_t*)&sessionstart ;                         
     sessionstart = time(NULL) ;                                        
     cleanstart = TRUE ; /* pppd ist started by this pcp-session */     
     strcpy(timestring,ctime(sessionstartptr)) ;                        
     if (appResources.verbose)
       syslog(LOG_INFO, "Timerstart: ***> %s <***", timestring);          
     else
       syslog(LOG_INFO, "Timerstart.", timestring);          

     system(pppCmdStr);
     if (appResources.verbose)
       syslog(LOG_INFO, "Executing: %s", pppCmdStr);
     timerhandleredial =
       XtAppAddTimeOut(appContext, appResources.redialInterval, 
		       redialCB, (XtPointer)clientData);
     sessionstartptr = (time_t*)&sessionstart ;                         
     sessionstart = time(NULL) ;                                        
     strcpy(timestring, ctime(sessionstartptr)) ;                        
     if (appResources.verbose)
       syslog(LOG_INFO, "%s: Redialinterval starts counting now...", timestring);          
   }  
   else
     if (pppPID == 0 && !stop)
     {
       timerhandleredial =
	 XtAppAddTimeOut(appContext, appResources.redialInterval, 
			 redialCB, (XtPointer)clientData);
       if (appResources.verbose)
	 syslog(LOG_INFO, "Restarting redial.");          
     }
}

void hupButtonCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   Widget dialog;

   XtVaSetValues(hupButton, XmNsensitive, FALSE, NULL);         
   /* warn the user before hanging up */                                 
   dialog = XmCreateQuestionDialog(appShell, "hupDlg", NULL, 0);
   XtDestroyWidget(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
   XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON),
		 XmNactivateCallback, &doHupCB, (XtPointer)appShell);
   XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON),
		 XmNactivateCallback, &dontHupCB, (XtPointer)appShell);
   XtManageChild(dialog);
}

void exitButtonCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   Widget dialog;

   /* if a connection exists, warn, else exit */
   if (connection || chatPID != 0) {
      dialog = XmCreateWarningDialog(appShell, "exitDlg", NULL, 0);
      XtDestroyWidget(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
      XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON),
		    XmNactivateCallback, &doExitCB, (XtPointer)appShell);
      XtManageChild(dialog);
   } 
   else 
     {
       remove(appResources.lockFile);
       remove(appResources.whoFile);
       XtAppSetExitFlag(appContext);  
     }
}

void breakButtonCB(Widget initiator,
		   XtPointer clientData,
		   XtPointer callData)
{
  Widget dialog;

  XtVaSetValues(breakButton, XmNsensitive, FALSE, NULL);         
  dialog = XmCreateQuestionDialog(appShell, "brkDlg", NULL, 0);
  XtDestroyWidget(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON),
		XmNactivateCallback, &doBreakCB, (XtPointer)appShell);
  XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON),
		XmNactivateCallback, &dontBreakCB, (XtPointer)appShell);
  XtManageChild(dialog);
}

void doExitCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   /* stop redial */
   RemoveRedialTimeout();
   remove(appResources.lockFile);
   remove(appResources.whoFile);
   XtAppSetExitFlag(appContext);
}

void doRunCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   XtAppSetExitFlag(appContext);
}

void doHupCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
   /* stop redial */
   RemoveRedialTimeout();
   if (pppPID != 0) {                                            
     syslog(LOG_NOTICE, "%s pppd.", strsignal(appResources.hupSignal));             
     kill(pppPID, appResources.hupSignal);                      
   }                                                            
   else                                                         
     {
       syslog(LOG_NOTICE, "Could not send signal to pppd.");             
       XtVaSetValues(hupButton, XmNsensitive, TRUE, NULL);         
     }
}                                                            

void dontHupCB(Widget initiator, 
		    XtPointer clientData, 
		    XtPointer callData)
{
  XtVaSetValues(hupButton, XmNsensitive, TRUE, NULL);         
}                                                            

void doBreakCB(Widget initiator,                                
	       XtPointer clientData,                            
	       XtPointer callData)                              
{                                                               
  RemoveRedialTimeout();
  /* turn off redial */
  if (connection) {                                              
    if (pppPID > 0) {                                            
      syslog(LOG_NOTICE, "%s pppd.", strsignal(appResources.hupSignal)); 
      kill(pppPID, appResources.hupSignal);                      
      syslog(LOG_NOTICE, "%s pppd.", strsignal(appResources.brkSignal)); 
      kill(pppPID, appResources.brkSignal);                      
      XtVaSetValues(dialButton, XmNsensitive, TRUE, NULL);          
      if (appResources.verbose)
	syslog(LOG_NOTICE, "Making sure lock-File is removed.");   
      remove(appResources.pidFile) ;
    }                                                            
    else                                                         
      syslog(LOG_NOTICE, "Could not send signal to pppd.");     
  }                                                              
  else {                                                         
    if (chatPID > 0) {                                           
      syslog(LOG_NOTICE, "%s chat.", strsignal(appResources.hupSignal));
      kill(chatPID, appResources.hupSignal);                     
      syslog(LOG_NOTICE, "%s chat.", strsignal(appResources.brkSignal));
      kill(chatPID, appResources.brkSignal);                     
      XtVaSetValues(dialButton, XmNsensitive, TRUE, NULL);          
      if (appResources.verbose)
	syslog(LOG_NOTICE, "Making sure ps-File is removed.");   
      remove(appResources.psFile) ;
    }                                                            
    else
      {                                                         
	syslog(LOG_NOTICE, "Could not send signal to chat.");     
      }
    if (pppPID > 0) {                                            
      syslog(LOG_NOTICE, "%s pppd.", strsignal(appResources.hupSignal));
      kill(pppPID, appResources.hupSignal);                      
      syslog(LOG_NOTICE, "%s pppd.", strsignal(appResources.brkSignal));
      kill(pppPID, appResources.brkSignal);                      
      XtVaSetValues(dialButton, XmNsensitive, TRUE, NULL);          
      if (appResources.verbose)
	syslog(LOG_NOTICE, "Making sure lock-File is removed.");   
      remove(appResources.pidFile) ;
    }                                                            
    else                                                         
      syslog(LOG_NOTICE, "Could not send signal to pppd.");     
  }
  stop = TRUE;
#if 0
  if (appResources.verbose)
    syslog(LOG_NOTICE, "Trying to send ATH to modem.");   
  if ((modem = fopen("/dev/modem","w")))
    fprintf(modem,"ATH\n");
  fclose(modem);
#endif
  XtVaSetValues(breakButton, XmNsensitive, TRUE, NULL);          
}                                                               

void dontBreakCB(Widget initiator,                                
	       XtPointer clientData,                            
	       XtPointer callData)                              
{                                                               
 XtVaSetValues(breakButton, XmNsensitive, TRUE, NULL);          
 XtVaSetValues(dialButton, XmNsensitive, TRUE, NULL);          
}                                                               

void updateLogCB(XtPointer clientData, XtIntervalId *timerId)
{
   char     wholine[MAX_LEN];
   char     line[MAX_LEN];

   if (connection) {
      /* do the animation, yeah! */                                  
      if (++flic >= MAX_ANIM) flic = 0;                              
      XtVaSetValues(animation, XmNlabelPixmap, anim[flic], NULL);   
   }

   /* update button status */
   XtVaSetValues(dialButton, XmNsensitive,!connection&&(chatPID == 0), NULL);
   XtVaSetValues(hupButton, XmNsensitive, connection, NULL);
   
   /* read new lines from log file until EOF and store them */
   if (logFile) { 
   while (fgets(line, MAX_LEN, logFile)) {
	 bufSize += strlen(line);
	 buffer = (char*)realloc(buffer, bufSize);
	 strcat(buffer, line);
	 updatelog = 1;
   }
   }

   system(whoCmdStr);
   /* prepare wholog buffer */
   whobuffer = (char*)malloc(whobufSize = 1);
   *whobuffer = '\0';

   if ((whoFile = fopen(appResources.whoFile, "r")) != NULL) {     
   while (fgets(wholine, MAX_LEN, whoFile)) {
     whobufSize += strlen(wholine);
     whobuffer = (char*)realloc(whobuffer, whobufSize);
     strcat(whobuffer, wholine);

     updatewho = 1;
   }
   fclose(whoFile); }
   else
     {
      strcpy(wholine,"Could not open wholog!");
      whobufSize += strlen(wholine);
      whobuffer = (char*)realloc(whobuffer, whobufSize);
      strcat(whobuffer, wholine);
      updatewho = 1;
     }
   
   /* update the value of the text widgets if necessary */
   if (updatelog) {
      XmTextSetString(logText, buffer);
     
      /* Scroll to the end of the text; this is a "shoot into 
       * the foot" -- but does s.o. know a nicer method? */
      /* I do ... :*/
      XtVaSetValues(logText, XmNcursorPosition, bufSize, NULL);
      updatelog = 0;
   }
   if (updatewho) {
      XmTextSetString(whoText, whobuffer);
      XtVaSetValues(whoText, XmNcursorPosition, whobufSize, NULL);

      updatewho = 0;
   }
   
   /* Tell Xt to call this callback again after a few millisecs */
   timerhandlelog =
     XtAppAddTimeOut(appContext, appResources.logInterval, 
		   updateLogCB, (XtPointer)appShell);
}

void updatetimerCB(XtPointer clientData, XtIntervalId *timerId)
{
   XmString timerXmstr;
   FILE    *pidFile;
   FILE    *psFile;
   int      h1, m1, s1 ;
   char    timerstringl[50];

   chatPID = 0;
   pppPID = 0;

   /* check if we are connected respectively if a pppd is running and
    * appropriately fetch the pppd's PID from it's temp file. */
   if ((connection = ((pidFile = fopen(appResources.pidFile, "r"))) != NULL)) {
      if (fscanf(pidFile, "%d", &pppPID) == EOF) pppPID = 0;       
      fclose(pidFile);
       /* check if pppd or chat are already */
       /* running to prevent random elapsed time */
	if (!cleanstart) {                                          
         sessionstart = time(NULL) ;                                
         cleanstart = TRUE ;                                        
	}                                                           
      /* update elapsed time: */                                    
      h1 = h ;                                                      
      m1 = m ;                                                      
      s1 = etime ;                                                  
      now = time(NULL) ;                                            
      etime = difftime(now, sessionstart) ;                         
      h = (unsigned int)(etime / 3600.);                            
      etime = etime - h * 3600 ;                                    
      m = (int)(etime / 60.);                                       
      etime = (etime - m * 60) ;                                    
      if ((h != h1) || (m != m1) || (etime != s1)) {                
      sprintf(timerstring, "%02d:%02d:%02d", h, m, (unsigned int)etime);
      timerXmstr = XmStringCreate (timerstring, XmSTRING_DEFAULT_CHARSET);
      XtSetArg(args[0], XmNlabelString, timerXmstr);                
      XtSetValues(timer, args, 1);                                  
      XmStringFree(timerXmstr);
      }                                                              
   }

   /* fetch chat-process PID and write it to psFile */              
   if (!connection) {                                               
      system(psCmdStr);                                             
      if ((psFile = fopen(appResources.psFile, "r")) != NULL)
      {     
      if (fscanf(psFile, "%d", &chatPID) == EOF) chatPID = 0;       
      else {                                                        
       /* check if pppd or chat are already */
       /* running to prevent random elapsed time */
	if (!cleanstart) {                                          
         sessionstart = time(NULL) ;                                
         cleanstart = TRUE ;                                        
	}                                                           
       /* update elapsed time: */                                   
       h1 = h ;                                                     
       m1 = m ;                                                     
       s1 = etime ;                                                 
       now = time(NULL) ;                                           
       etime = difftime(now, sessionstart) ;                        
       h = (unsigned int)(etime / 3600.);                           
       etime = etime - h * 3600 ;                                   
       m = (int)(etime / 60.);                                      
       etime = (etime - m * 60) ;                                   
       if ((h != h1) || (m != m1) || (etime != s1)) {                
       sprintf(timerstring, "%02d:%02d:%02d", h, m, (unsigned int)etime);
       timerXmstr = XmStringCreate (timerstring, XmSTRING_DEFAULT_CHARSET); 
        XtSetArg(args[0], XmNlabelString, timerXmstr);               
        XtSetValues(timer, args, 1);                                 
	XmStringFree(timerXmstr);
       }                                                            
      }                                                             
      fclose(psFile);                                               
      }                                                             
      else                                                          
       syslog(LOG_NOTICE, "Failed to get chat-PID.");               
   }

   /* update button status */
   XtVaSetValues(dialButton,XmNsensitive,!connection&&(chatPID == 0), NULL);
   XtVaSetValues(hupButton, XmNsensitive, connection, NULL);
  
   strcpy(timerstringl, timerstring);
   if (!connection)
     {
       /* Change Windowtitle */
       strcat(timerstringl, " PCP 0.9.5 (not connected)\0");
       timerXmstr = XmStringGenerate(timerstringl, (XmStringTag)NULL,
				     XmCHARSET_TEXT, (XmStringTag)NULL);
       XmeSetWMShellTitle(timerXmstr, appShell);
       XmStringFree(timerXmstr);
     }
   else
     {
       /* Change Windowtitle */
       strcat(timerstringl, " PCP 0.9.5 (connected)\0");
       timerXmstr = XmStringGenerate(timerstringl, (XmStringTag)NULL,
				    XmCHARSET_TEXT, (XmStringTag)NULL);
       XmeSetWMShellTitle(timerXmstr, appShell);
       XmStringFree(timerXmstr);
     } 
   /* Tell Xt to call this callback again after a few millisecs */
   timerhandletimer =
     XtAppAddTimeOut(appContext, appResources.timerInterval, 
		     updatetimerCB, (XtPointer)appShell);
}
